@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+    Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "config.h"
 
@@ -43,15 +43,15 @@ const char *myname;
    so that error messages from ck_fread, ck_fwrite, etc. can print the
    name of the file that had the error */
 
-struct id
+struct open_file
   {
     FILE *fp;
     char *name;
-    struct id *link;
+    struct open_file *link;
     unsigned temp : 1;
   };
 
-static struct id *utils_id_s = NULL;
+static struct open_file *open_files = NULL;
 static void do_ck_fclose P_((FILE *fp));
 
 /* Print an error message and exit */
@@ -86,34 +86,34 @@ panic(const char *str, ...)
   putc('\n', stderr);
 
   /* Unlink the temporary files.  */
-  while (utils_id_s)
+  while (open_files)
     {
-      if (utils_id_s->temp)
+      if (open_files->temp)
 	{
-	  int fd = fileno (utils_id_s->fp);
-	  fclose (utils_id_s->fp);
+	  int fd = fileno (open_files->fp);
+	  fclose (open_files->fp);
 	  errno = 0;
-	  unlink (utils_id_s->name);
+	  unlink (open_files->name);
           if (errno != 0)
-            fprintf (stderr, _("cannot remove %s: %s"), utils_id_s->name, strerror (errno));
+            fprintf (stderr, _("cannot remove %s: %s"), open_files->name, strerror (errno));
 	}
 
-      utils_id_s = utils_id_s->link;
+      open_files = open_files->link;
     }
 
   exit(4);
 }
 
 
-/* Internal routine to get a filename from utils_id_s */
+/* Internal routine to get a filename from open_files */
 static const char *utils_fp_name P_((FILE *fp));
 static const char *
 utils_fp_name(fp)
   FILE *fp;
 {
-  struct id *p;
+  struct open_file *p;
 
-  for (p=utils_id_s; p; p=p->link)
+  for (p=open_files; p; p=p->link)
     if (p->fp == fp)
       return p->name;
   if (fp == stdin)
@@ -134,7 +134,7 @@ ck_fopen(name, mode, fail)
   bool fail;
 {
   FILE *fp;
-  struct id *p;
+  struct open_file *p;
 
   fp = fopen (name, mode);
   if (!fp)
@@ -145,7 +145,7 @@ ck_fopen(name, mode, fail)
       return NULL;
     }
 
-  for (p=utils_id_s; p; p=p->link)
+  for (p=open_files; p; p=p->link)
     {
       if (fp == p->fp)
 	{
@@ -155,9 +155,9 @@ ck_fopen(name, mode, fail)
     }
   if (!p)
     {
-      p = MALLOC(1, struct id);
-      p->link = utils_id_s;
-      utils_id_s = p;
+      p = MALLOC(1, struct open_file);
+      p->link = open_files;
+      open_files = p;
     }
   p->name = ck_strdup(name);
   p->fp = fp;
@@ -173,7 +173,7 @@ ck_mkstemp (p_filename, tmpdir, base)
   char *template;
   FILE *fp;
   int fd;
-  struct id *p;
+  struct open_file *p;
 
   if (tmpdir == NULL)
     tmpdir = getenv("TMPDIR");
@@ -198,12 +198,12 @@ ck_mkstemp (p_filename, tmpdir, base)
   *p_filename = template;
   fp = fdopen (fd, "w");
 
-  p = MALLOC(1, struct id);
+  p = MALLOC(1, struct open_file);
   p->name = ck_strdup (template);
   p->fp = fp;
   p->temp = true;
-  p->link = utils_id_s;
-  utils_id_s = p;
+  p->link = open_files;
+  open_files = p;
   return fp;
 }
 
@@ -268,12 +268,12 @@ void
 ck_fclose(stream)
   FILE *stream;
 {
-  struct id r;
-  struct id *prev;
-  struct id *cur;
+  struct open_file r;
+  struct open_file *prev;
+  struct open_file *cur;
 
   /* a NULL stream means to close all files */
-  r.link = utils_id_s;
+  r.link = open_files;
   prev = &r;
   while ( (cur = prev->link) )
     {
@@ -288,7 +288,7 @@ ck_fclose(stream)
 	prev = cur;
     }
 
-  utils_id_s = r.link;
+  open_files = r.link;
 
   /* Also care about stdout, because if it is redirected the
      last output operations might fail and it is important
