@@ -1,5 +1,5 @@
 /*  GNU SED, a batch stream editor.
-    Copyright (C) 1989,90,91,92,93,94,95,98,99,2002,2003
+    Copyright (C) 1989,90,91,92,93,94,95,98,99,2002,2003,2004,2005,2006
     Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+    Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 /* compile.c: translate sed source into internal form */
 
@@ -463,7 +463,7 @@ snarf_char_class(b, cur_stat)
   ch = inchar();
   if (ch == '^')
     ch = add_then_next(b, ch);
-  else if (ch == CLOSE_BRACKET)
+  if (ch == CLOSE_BRACKET)
     ch = add_then_next(b, ch);
   
   /* States are:
@@ -1261,7 +1261,7 @@ compile_program(vector)
 
 	    flags = mark_subst_opts(cur_cmd->x.cmd_subst);
 	    cur_cmd->x.cmd_subst->regx =
-	      compile_regex(b, flags, cur_cmd->x.cmd_subst->max_id);
+	      compile_regex(b, flags, cur_cmd->x.cmd_subst->max_id + 1);
 	    free_buffer(b);
 	  }
 	  break;
@@ -1274,12 +1274,12 @@ compile_program(vector)
             char *src_buf, *dest_buf;
   
 	    slash = inchar();
-	    if ( !(b = match_slash(slash, true)) )
+	    if ( !(b = match_slash(slash, false)) )
 	      bad_prog(_(UNTERM_Y_CMD));
             src_buf = get_buffer(b);
 	    len = normalize_text(src_buf, size_buffer (b), TEXT_BUFFER);
 
-            if ( !(b2 = match_slash(slash, true)) )
+            if ( !(b2 = match_slash(slash, false)) )
  	      bad_prog(_(UNTERM_Y_CMD));
             dest_buf = get_buffer(b2);
 	    dest_len = normalize_text(dest_buf, size_buffer (b2), TEXT_BUFFER);
