@@ -1,6 +1,5 @@
-
 /*  GNU SED, a batch stream editor.
-    Copyright (C) 1989,90,91,92,93,94,95,98,99,2002,2003
+    Copyright (C) 1989,90,91,92,93,94,95,98,99,2002,2003,2004,2005,2006
     Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -15,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+    Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #undef EXPERIMENTAL_DASH_N_OPTIMIZATION	/*don't use -- is very buggy*/
 #define INITIAL_BUFFER_SIZE	50
@@ -738,8 +737,7 @@ reset_addresses(vec)
 
   for (cur_cmd = vec->v, n = vec->v_length; n--; cur_cmd++)
     if (cur_cmd->a1
-	&& (cur_cmd->a1->addr_type == ADDR_IS_NUM
-	    || cur_cmd->a1->addr_type == ADDR_IS_NUM_MOD)
+	&& cur_cmd->a1->addr_type == ADDR_IS_NUM
 	&& cur_cmd->a1->addr_number == 0)
       cur_cmd->range_state = RANGE_ACTIVE;
     else
@@ -1466,6 +1464,7 @@ execute_program(vec, input)
             case 'q':
               if (!no_default_output)
                 output_line(line.active, line.length, line.chomped, &output_file);
+	      dump_append_queue();
 
 	    case 'Q':
 	      return cur_cmd->x.int_arg == -1 ? 0 : cur_cmd->x.int_arg;
@@ -1708,10 +1707,14 @@ process_files(the_program, argv)
 #ifdef EXPERIMENTAL_DASH_N_OPTIMIZATION
   branches = count_branches(the_program);
 #endif /*EXPERIMENTAL_DASH_N_OPTIMIZATION*/
-  input.file_list = stdin_argv;
   input.reset_at_next_file = true;
   if (argv && *argv)
     input.file_list = argv;
+  else if (in_place_extension)
+    panic(_("no input files"));
+  else
+    input.file_list = stdin_argv;
+
   input.bad_count = 0;
   input.line_number = 0;
   input.read_fn = read_always_fail;
