@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+    Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 
 #include "sed.h"
@@ -45,9 +45,6 @@
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
-#ifdef HAVE_LOCALE_H
-# include <locale.h>
-#endif
 #include "getopt.h"
 
 #ifndef BOOTSTRAP
@@ -102,7 +99,7 @@ usage(status)
 Usage: %s [OPTION]... {script-only-if-no-other-script} [input-file]...\n\
 \n"), myname);
 
-  fprintf(out, _("-n, --quiet, --silent\n\
+  fprintf(out, _("  -n, --quiet, --silent\n\
                  suppress automatic printing of pattern space\n"));
   fprintf(out, _("  -e script, --expression=script\n\
                  add the script to the commands to be executed\n"));
